@@ -152,7 +152,7 @@ unur_distr_condi_new( const struct unur_distr *distr, const double *pos, const d
   memset( ar, 0, distr->dim * sizeof(double) );
   if ( (unur_distr_cont_set_pdfparams_vec( condi, iXARG, ar, distr->dim ) != UNUR_SUCCESS) ||
        (unur_distr_cont_set_pdfparams_vec( condi, iGRADF, ar, distr->dim ) != UNUR_SUCCESS) ) {
-    _unur_distr_free(condi); return NULL; 
+    _unur_distr_free(condi); free(ar); return NULL; 
   }
   free(ar);
 
@@ -235,8 +235,8 @@ unur_distr_condi_set_condition( struct unur_distr *condi, const double *pos, con
     }
     else {
       /* arbitrary direction: we ignore given domain */
-      CONDI.trunc[0] = CONDI.domain[0] = -INFINITY;
-      CONDI.trunc[1] = CONDI.domain[1] = INFINITY;
+      CONDI.trunc[0] = CONDI.domain[0] = -UNUR_INFINITY;
+      CONDI.trunc[1] = CONDI.domain[1] = UNUR_INFINITY;
     }
   }
 
