@@ -14,7 +14,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2000-2006 Wolfgang Hoermann and Josef Leydold             *
+ *   Copyright (c) 2000-2010  Wolfgang Hoermann and Josef Leydold            *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -37,7 +37,6 @@
 /*---------------------------------------------------------------------------*/
 
 #include <unur_source.h>
-#include <specfunct/unur_specfunct_source.h>
 #include <distributions/unur_stddistr.h>
 #include "distr.h"
 #include "cont.h"
@@ -103,7 +102,7 @@ unur_distr_corder_new( const struct unur_distr *distr, int n, int k )
   COOKIE_CHECK(distr,CK_DISTR_CONT,NULL);
 
   if (distr->id == UNUR_DISTR_CORDER) {
-    _unur_warning(distr_name,UNUR_ERR_DISTR_INVALID,"No order statistics of order statistics allowed");
+    _unur_error(distr_name,UNUR_ERR_DISTR_INVALID,"No order statistics of order statistics allowed");
     return NULL; 
   }
 
@@ -218,7 +217,7 @@ unur_distr_corder_set_rank( struct unur_distr *os, int n, int k )
 
   /* check parameters n and k */
   if (n < 2 || k < 1 || k > n) {
-    _unur_warning(distr_name,UNUR_ERR_DISTR_SET,"n < 2 or k < 1 or k > n");
+    _unur_error(distr_name,UNUR_ERR_DISTR_SET,"n < 2 or k < 1 or k > n");
     return UNUR_ERR_DISTR_SET;
   }
 
@@ -295,12 +294,12 @@ _unur_pdf_corder( double x, const struct unur_distr *os )
   double p,q;   /* shape parameters for beta distribution */
 
   /* check arguments */
-  _unur_check_NULL( NULL, os, INFINITY );
-  _unur_check_distr_object( os, CONT, INFINITY );
-  CHECK_NULL( os->base, INFINITY );
-  _unur_check_distr_object( os->base, CONT, INFINITY );
-  CHECK_NULL( os->base->data.cont.cdf, INFINITY );
-  CHECK_NULL( os->base->data.cont.pdf, INFINITY );
+  _unur_check_NULL( NULL, os, UNUR_INFINITY );
+  _unur_check_distr_object( os, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base, UNUR_INFINITY );
+  _unur_check_distr_object( os->base, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.cdf, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.pdf, UNUR_INFINITY );
 
   Fx = (*(os->base->data.cont.cdf)) (x, os->base);
   fx = (*(os->base->data.cont.pdf)) (x, os->base);
@@ -336,13 +335,13 @@ _unur_dpdf_corder( double x, const struct unur_distr *os )
   double lFx, lFy;
 
   /* check arguments */
-  _unur_check_NULL( NULL, os, INFINITY );
-  _unur_check_distr_object( os, CONT, INFINITY );
-  CHECK_NULL( os->base, INFINITY );
-  _unur_check_distr_object( os->base, CONT, INFINITY );
-  CHECK_NULL( os->base->data.cont.cdf, INFINITY );
-  CHECK_NULL( os->base->data.cont.pdf, INFINITY );
-  CHECK_NULL( os->base->data.cont.dpdf, INFINITY );
+  _unur_check_NULL( NULL, os, UNUR_INFINITY );
+  _unur_check_distr_object( os, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base, UNUR_INFINITY );
+  _unur_check_distr_object( os->base, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.cdf, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.pdf, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.dpdf, UNUR_INFINITY );
 
   Fx = (*(os->base->data.cont.cdf)) (x, os->base);
   fx = (*(os->base->data.cont.pdf)) (x, os->base);
@@ -382,11 +381,11 @@ _unur_cdf_corder( double x, const struct unur_distr *os )
   double p,q;   /* shape parameters for beta distribution */
 
   /* check arguments */
-  _unur_check_NULL( NULL, os, INFINITY );
-  _unur_check_distr_object( os, CONT, INFINITY );
-  CHECK_NULL( os->base, INFINITY );
-  _unur_check_distr_object( os->base, CONT, INFINITY );
-  CHECK_NULL( os->base->data.cont.cdf, INFINITY );
+  _unur_check_NULL( NULL, os, UNUR_INFINITY );
+  _unur_check_distr_object( os, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base, UNUR_INFINITY );
+  _unur_check_distr_object( os->base, CONT, UNUR_INFINITY );
+  CHECK_NULL( os->base->data.cont.cdf, UNUR_INFINITY );
 
   Fx = (*(os->base->data.cont.cdf)) (x, os->base);
 
@@ -394,7 +393,7 @@ _unur_cdf_corder( double x, const struct unur_distr *os )
   q = OS.params[0] - OS.params[1] + 1.;   /* n-k+1 */
 
   /* CDF(x) = B(F(x)) */
-  return _unur_sf_incomplete_beta(Fx,p,q);
+  return _unur_SF_incomplete_beta(Fx,p,q);
 
 } /* end of _unur_cdf_corder() */
 
@@ -405,10 +404,10 @@ _unur_upd_area_corder( UNUR_DISTR *os )
 {
 
   /* log of normalization constant */
-  /* LOGNORMCONSTANT = _unur_sf_ln_gamma(k) + _unur_sf_ln_gamma(n-k+1) - _unur_sf_ln_gamma(n+1); */
-  LOGNORMCONSTANT = ( _unur_sf_ln_gamma(OS.params[1]) 
-		      + _unur_sf_ln_gamma(OS.params[0] - OS.params[1] + 1.) 
-		      - _unur_sf_ln_gamma(OS.params[0] + 1.) );
+  /* LOGNORMCONSTANT = _unur_SF_ln_gamma(k) + _unur_SF_ln_gamma(n-k+1) - _unur_SF_ln_gamma(n+1); */
+  LOGNORMCONSTANT = ( _unur_SF_ln_gamma(OS.params[1]) 
+		      + _unur_SF_ln_gamma(OS.params[0] - OS.params[1] + 1.) 
+		      - _unur_SF_ln_gamma(OS.params[0] + 1.) );
 
   /* we assume that the PDF is the derivative of the given CDF ! */
 
@@ -420,8 +419,8 @@ _unur_upd_area_corder( UNUR_DISTR *os )
       /* no CDF */
       return UNUR_ERR_DISTR_REQUIRED;
 
-    OS.area  = (OS.domain[1] < INFINITY)  ? _unur_cdf_corder(OS.domain[1],os) : 1.;
-    OS.area -= (OS.domain[0] > -INFINITY) ? _unur_cdf_corder(OS.domain[0],os) : 0.;
+    OS.area  = (OS.domain[1] < UNUR_INFINITY)  ? _unur_cdf_corder(OS.domain[1],os) : 1.;
+    OS.area -= (OS.domain[0] > -UNUR_INFINITY) ? _unur_cdf_corder(OS.domain[0],os) : 0.;
   }
 
   return (OS.area > 0.) ? UNUR_SUCCESS : UNUR_ERR_DISTR_DATA;
