@@ -135,6 +135,7 @@ unur_distr_discr_new( void )
   /* probability mass function */
   DISTR.pmf       = NULL;          /* pointer to PMF                         */
   DISTR.cdf       = NULL;          /* pointer to CDF                         */
+  DISTR.invcdf    = NULL;          /* pointer to inverse CDF                 */
 
   DISTR.init      = NULL;          /* pointer to special init routine        */
 
@@ -389,7 +390,7 @@ unur_distr_discr_make_pv( struct unur_distr *distr )
     cdf = 0.;       /* cumulated sum of PV                   */
     cdf_old = 0.;   /* cumulated sum of PV in last iteration */
     /* threshold for truncating PV */
-    thresh_cdf = (distr->set & UNUR_DISTR_SET_PMFSUM) ? (1.-1.e-8)*DISTR.sum : INFINITY;
+    thresh_cdf = (distr->set & UNUR_DISTR_SET_PMFSUM) ? (1.-1.e-8)*DISTR.sum : UNUR_INFINITY;
 
     /* compute PV */
     for (n_alloc = size_alloc; n_alloc <= max_alloc; n_alloc += size_alloc) {
@@ -484,8 +485,8 @@ unur_distr_discr_eval_pv( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   if (DISTR.pv != NULL) {
     /* use probability vector */
@@ -507,8 +508,8 @@ unur_distr_discr_eval_pv( int k, const struct unur_distr *distr )
   }
 
   /* else: data missing */
-  _unur_warning(distr->name,UNUR_ERR_DISTR_DATA,"");
-  return INFINITY;
+  _unur_error(distr->name,UNUR_ERR_DISTR_DATA,"");
+  return UNUR_INFINITY;
 
 } /* end of unur_distr_discr_eval_pv() */
 
@@ -541,7 +542,7 @@ unur_distr_discr_set_pmf( struct unur_distr *distr, UNUR_FUNCT_DISCR *pmf )
 
   /* we do not allow overwriting a PMF */
   if (DISTR.pmf != NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of PMF not allowed");
+    _unur_error(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of PMF not allowed");
     return UNUR_ERR_DISTR_SET;
   }
 
@@ -583,7 +584,7 @@ unur_distr_discr_set_cdf( struct unur_distr *distr, UNUR_FUNCT_DISCR *cdf )
 
   /* we do not allow overwriting a CDF */
   if (DISTR.cdf != NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of CDF not allowed");
+    _unur_error(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of CDF not allowed");
     return UNUR_ERR_DISTR_SET;
   }
 
@@ -597,6 +598,44 @@ unur_distr_discr_set_cdf( struct unur_distr *distr, UNUR_FUNCT_DISCR *cdf )
 
 /*---------------------------------------------------------------------------*/
 
+int
+unur_distr_discr_set_invcdf( struct unur_distr *distr, UNUR_IFUNCT_DISCR *invcdf )
+     /*----------------------------------------------------------------------*/
+     /* set inverse CDF of distribution                                      */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   distr  ... pointer to distribution object                          */
+     /*   invcdf ... pointer to inverse CDF                                  */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   UNUR_SUCCESS ... on success                                        */
+     /*   error code   ... on error                                          */
+     /*----------------------------------------------------------------------*/
+{
+  /* check arguments */
+  _unur_check_NULL( NULL, distr, UNUR_ERR_NULL );
+  _unur_check_NULL( distr->name, invcdf,UNUR_ERR_NULL );
+  _unur_check_distr_object( distr, DISCR, UNUR_ERR_DISTR_INVALID );
+  
+  /* we do not allow overwriting an inverse cdf */
+  if (DISTR.invcdf != NULL) {
+    _unur_error(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of inverse CDF not allowed");
+    return UNUR_ERR_DISTR_SET;
+  }
+
+  /* for derived distributions (e.g. order statistics) not possible */
+  if (distr->base) return UNUR_ERR_DISTR_INVALID;
+
+  /* changelog */
+  distr->set &= ~UNUR_DISTR_SET_MASK_DERIVED;
+  /* derived parameters like mode, area, etc. might be wrong now! */
+
+  DISTR.invcdf = invcdf;
+  return UNUR_SUCCESS;
+} /* end of unur_distr_discr_set_invcdf() */
+
+/*---------------------------------------------------------------------------*/
+
 UNUR_FUNCT_DISCR *
 unur_distr_discr_get_pmf( const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
@@ -639,6 +678,27 @@ unur_distr_discr_get_cdf( const struct unur_distr *distr )
 
 /*---------------------------------------------------------------------------*/
 
+UNUR_IFUNCT_DISCR *
+unur_distr_discr_get_invcdf( const struct unur_distr *distr )
+     /*----------------------------------------------------------------------*/
+     /* get pointer to inverse CDF of distribution                           */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   distr ... pointer to distribution object                           */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   pointer to inverse CDF                                             */
+     /*----------------------------------------------------------------------*/
+{
+  /* check arguments */
+  _unur_check_NULL( NULL, distr, NULL );
+  _unur_check_distr_object( distr, DISCR, NULL );
+
+  return DISTR.invcdf;
+} /* end of unur_distr_discr_get_invcdf() */
+
+/*---------------------------------------------------------------------------*/
+
 double
 unur_distr_discr_eval_pmf( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
@@ -653,12 +713,12 @@ unur_distr_discr_eval_pmf( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   if (DISTR.pmf == NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_DATA,"");
-    return INFINITY;
+    _unur_error(distr->name,UNUR_ERR_DISTR_DATA,"");
+    return UNUR_INFINITY;
   }
 
   return _unur_discr_PMF(k,distr);
@@ -680,12 +740,12 @@ unur_distr_discr_eval_cdf( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   if (DISTR.cdf == NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_DATA,"");
-    return INFINITY;
+    _unur_error(distr->name,UNUR_ERR_DISTR_DATA,"");
+    return UNUR_INFINITY;
   }
 
   return _unur_discr_CDF(k,distr);
@@ -693,6 +753,40 @@ unur_distr_discr_eval_cdf( int k, const struct unur_distr *distr )
 
 /*---------------------------------------------------------------------------*/
 
+int
+unur_distr_discr_eval_invcdf( double u, const struct unur_distr *distr )
+     /*----------------------------------------------------------------------*/
+     /* evaluate inverse CDF of distribution at u                            */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   u     ... argument for inverse CDF                                 */
+     /*   distr ... pointer to distribution object                           */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   invcdf(u)                                                          */
+     /*   INT_MAX    ... on error                                            */
+     /*----------------------------------------------------------------------*/
+{
+  /* check arguments */
+  _unur_check_NULL( NULL, distr, INT_MAX );
+  _unur_check_distr_object( distr, DISCR, INT_MAX );
+
+  if (DISTR.invcdf == NULL) {
+    _unur_error(distr->name,UNUR_ERR_DISTR_DATA,"");
+    return INT_MAX;
+  }
+
+  if (u<=0.)
+    return DISTR.domain[0];
+  if (u>=1.)
+    return DISTR.domain[1];
+  else
+    return _unur_discr_invCDF(u,distr);
+
+} /* end of unur_distr_discr_eval_invcdf() */
+
+/*---------------------------------------------------------------------------*/
+
 int
 unur_distr_discr_set_pmfstr( struct unur_distr *distr, const char *pmfstr )
      /*----------------------------------------------------------------------*/
@@ -720,7 +814,7 @@ unur_distr_discr_set_pmfstr( struct unur_distr *distr, const char *pmfstr )
 
   /* we do not allow overwriting a PMF */
   if (DISTR.pmf != NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of PMF not allowed");
+    _unur_error(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of PMF not allowed");
     return UNUR_ERR_DISTR_SET;
   }
 
@@ -764,7 +858,7 @@ unur_distr_discr_set_cdfstr( struct unur_distr *distr, const char *cdfstr )
 
   /* we do not allow overwriting a CDF */
   if (DISTR.cdf != NULL) {
-    _unur_warning(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of CDF not allowed");
+    _unur_error(distr->name,UNUR_ERR_DISTR_SET,"Overwriting of CDF not allowed");
     return UNUR_ERR_DISTR_SET;
   }
 
@@ -803,8 +897,8 @@ _unur_distr_discr_eval_pmf_tree( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   return ((DISTR.pmftree) ? _unur_fstr_eval_tree(DISTR.pmftree,(double)k) : 0.);
 } /* end of _unur_distr_discr_eval_pmf_tree() */
@@ -825,8 +919,8 @@ _unur_distr_discr_eval_cdf_tree( int k, const struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   return ((DISTR.cdftree) ? _unur_fstr_eval_tree(DISTR.cdftree,(double)k) : 0.);
 } /* end of _unur_distr_discr_eval_cdf_tree() */
@@ -1034,7 +1128,7 @@ unur_distr_discr_get_domain( const struct unur_distr *distr, int *left, int *rig
      /*   if no boundaries have been set [INT_MIN, INT_MAX] is returned.     */
      /*----------------------------------------------------------------------*/
 {
-  /* in case of error the boundaries are set to +/- INFINITY */
+  /* in case of error the boundaries are set to +/- UNUR_INFINITY */
   *left = INT_MIN;
   *right = INT_MAX;
 
@@ -1276,15 +1370,15 @@ unur_distr_discr_get_pmfsum( struct unur_distr *distr )
      /*----------------------------------------------------------------------*/
 {
   /* check arguments */
-  _unur_check_NULL( NULL, distr, INFINITY );
-  _unur_check_distr_object( distr, DISCR, INFINITY );
+  _unur_check_NULL( NULL, distr, UNUR_INFINITY );
+  _unur_check_distr_object( distr, DISCR, UNUR_INFINITY );
 
   /* sum known ? */
   if ( !(distr->set & UNUR_DISTR_SET_PMFSUM) ) {
     /* try to compute sum */
     if ( unur_distr_discr_upd_pmfsum(distr) != UNUR_SUCCESS ) {
       _unur_error(distr->name,UNUR_ERR_DISTR_GET,"sum");
-      return INFINITY;
+      return UNUR_INFINITY;
     }
   }
 
