@@ -10,7 +10,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2000-2006 Wolfgang Hoermann and Josef Leydold             *
+ *   Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold             *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -35,7 +35,6 @@
 #include <unur_source.h>
 #include <methods/cstd.h>
 #include <methods/cstd_struct.h>
-#include <specfunct/unur_specfunct_source.h>
 #include "unur_distributions_source.h"
 
 /*---------------------------------------------------------------------------*/
@@ -49,7 +48,7 @@ inline static int beta_b01_init( struct unur_gen *gen );
 inline static int beta_b1prs_init( struct unur_gen *gen );
 
 /*---------------------------------------------------------------------------*/
- /* abbreviations */
+/* abbreviations */
 
 #define PAR       ((struct unur_cstd_par*)par->datap) /* data for parameter object */
 #define GEN       ((struct unur_cstd_gen*)gen->datap) /* data for generator object */
@@ -57,7 +56,7 @@ inline static int beta_b1prs_init( struct unur_gen *gen );
 
 #define uniform()  _unur_call_urng(gen->urng) /* call for uniform prng       */
 
-#define MAX_gen_params 22      /* maximal number of parameters for generator */
+/* #define MAX_gen_params (22)    maximal number of parameters for generator */
 
 #define p     (DISTR.params[0])   /* shape parameter */
 #define q     (DISTR.params[1])   /* shape parameter */
@@ -96,39 +95,36 @@ _unur_stdgen_beta_init( struct unur_par *par, struct unur_gen *gen )
   case 1:  /* Rejection with log-logistic envelopes */
     if (gen==NULL) return UNUR_SUCCESS; /* test existence only  */
     if (p>1. && q>1.) {
-      _unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_bb );
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_bb );
       return beta_bb_init( gen );
     }
     else {
-      _unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_bc );
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_bc );
       return beta_bc_init( gen );
     }
 
   case 2:  /* Stratified Rejection/Patchwork Rejection */
     if (gen==NULL) return UNUR_SUCCESS; /* test existence only  */ 
-    if (p>1.)
-      if (q>1.) {    /* p > 1 && q > 1 */
-	_unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_b1prs );
-	return beta_b1prs_init( gen );
-      }
-      else {         /* p > 1 && q <= 1 */
-	_unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_b01 );
-	return beta_b01_init( gen );
-      }
-    else
-      if (q>1.) {    /* p <= 1 && q > 1 */
-	_unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_b01 );
-	return beta_b01_init( gen );
-      }
-      else {         /* p <= 1 && q <= 1 */
-	_unur_cstd_set_sampling_routine( par,gen,_unur_stdgen_sample_beta_b00 );
-	return beta_b00_init( gen );
-      }
+    if (p > 1. && q > 1.) {
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b1prs );
+      return beta_b1prs_init( gen );
+    }
+    else if (p < 1. && q < 1.) {
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b00 );
+      return beta_b00_init( gen );
+    }
+    else if (_unur_isone(p) || _unur_isone(q)) { /* p==1 || q==1 */
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_binv );
+      return UNUR_SUCCESS;
+    }
+    else { /* (p > 1 && q < 1) || (p < 1 && q > 1) */
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b01 );
+      return beta_b01_init( gen );
+    }
 
-  case UNUR_STDGEN_INVERSION:   /* inversion method */
   default: /* no such generator */
-    if (gen) _unur_warning(gen->genid,UNUR_ERR_SHOULD_NOT_HAPPEN,"");
     return UNUR_FAILURE;
+
   }
 
 } /* end of _unur_stdgen_beta_init() */
@@ -168,15 +164,15 @@ _unur_stdgen_beta_init( struct unur_par *par, struct unur_gen *gen )
  *****************************************************************************/
 
 /*---------------------------------------------------------------------------*/
+#define GEN_N_PARAMS (8)
 #define am      (GEN->gen_param[0])
 #define bm      (GEN->gen_param[1])
 #define al      (GEN->gen_param[2])
 #define alnam   (GEN->gen_param[3])
 #define be      (GEN->gen_param[4])
-#define ga      (GEN->gen_param[5])
-#define si      (GEN->gen_param[6])
-#define rk1     (GEN->gen_param[7])
-#define rk2     (GEN->gen_param[8])
+#define si      (GEN->gen_param[5])
+#define rk1     (GEN->gen_param[6])
+#define rk2     (GEN->gen_param[7])
 /*---------------------------------------------------------------------------*/
 
 inline static int
@@ -187,9 +183,9 @@ beta_bc_init( struct unur_gen *gen )
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
 
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
 
   /* -X- setup code -X- */
@@ -216,8 +212,8 @@ _unur_stdgen_sample_beta_bc(  struct unur_gen *gen )
   double u1,u2,v,w,y,z;
 
   /* check arguments */
-  CHECK_NULL(gen,INFINITY);
-  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
 
   while (1) {
     /* Step 1 */
@@ -293,19 +289,36 @@ _unur_stdgen_sample_beta_bc(  struct unur_gen *gen )
 
 } /* end of _unur_stdgen_sample_beta_bc() */
 
+/*---------------------------------------------------------------------------*/
+#undef GEN_N_PARAMS
+#undef am
+#undef bm
+#undef al
+#undef alnam
+#undef be
+#undef si
+#undef rk1
+#undef rk2
+/*---------------------------------------------------------------------------*/
+#define GEN_N_PARAMS (5)
+#define am      (GEN->gen_param[0])
+#define bm      (GEN->gen_param[1])
+#define al      (GEN->gen_param[2])
+#define be      (GEN->gen_param[3])
+#define ga      (GEN->gen_param[4])
 /*---------------------------------------------------------------------------*/
 
 inline static int
 beta_bb_init( struct unur_gen *gen )
-     /* p > 1. && q > 1 */ 
+     /* p > 1. && q > 1. */ 
 {
   /* check arguments */
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
 
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
 
   /* -X- setup code -X- */
@@ -322,15 +335,15 @@ beta_bb_init( struct unur_gen *gen )
 
 double 
 _unur_stdgen_sample_beta_bb(  struct unur_gen *gen )
-     /* p > 1. && q > 1 */ 
+     /* p > 1. && q > 1. */ 
 {
   /* -X- generator code -X- */
   double X;
   double u1,u2,v,w,z,r,s,t;
 
   /* check arguments */
-  CHECK_NULL(gen,INFINITY);
-  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
 
   while (1) {
     /* Step 1 */
@@ -363,15 +376,12 @@ _unur_stdgen_sample_beta_bb(  struct unur_gen *gen )
 } /* end of _unur_stdgen_sample_beta_bb() */
 
 /*---------------------------------------------------------------------------*/
+#undef GEN_N_PARAMS
 #undef am
 #undef bm
 #undef al
-#undef alnam
 #undef be
 #undef ga
-#undef si
-#undef rk1
-#undef rk2
 /*---------------------------------------------------------------------------*/
 
 /*---------------------------------------------------------------------------*/
@@ -417,6 +427,7 @@ _unur_stdgen_sample_beta_bb(  struct unur_gen *gen )
  *****************************************************************************/
 
 /*---------------------------------------------------------------------------*/
+#define GEN_N_PARAMS (8)
 #define p_      (GEN->gen_param[0])
 #define q_      (GEN->gen_param[1])
 #define c       (GEN->gen_param[2])
@@ -429,15 +440,15 @@ _unur_stdgen_sample_beta_bb(  struct unur_gen *gen )
 
 inline static int
 beta_b00_init( struct unur_gen *gen )
-     /* p < 1. && q < 1 */ 
+     /* p < 1. && q < 1. */ 
 {
   /* check arguments */
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
 
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
 
   /* -X- setup code -X- */
@@ -458,14 +469,14 @@ beta_b00_init( struct unur_gen *gen )
 
 double 
 _unur_stdgen_sample_beta_b00(  struct unur_gen *gen )
-     /* p < 1. && q < 1 */
+     /* p < 1. && q < 1. */
 {
   /* -X- generator code -X- */
   double U, V, X, Z;
 
   /* check arguments */
-  CHECK_NULL(gen,INFINITY);
-  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
 
   while (1) {
     U = uniform() * p2;
@@ -505,6 +516,7 @@ _unur_stdgen_sample_beta_b00(  struct unur_gen *gen )
 } /* end of _unur_stdgen_sample_beta_b00() */
 
 /*---------------------------------------------------------------------------*/
+#undef GEN_N_PARAMS
 #undef p_
 #undef q_
 #undef c
@@ -514,6 +526,7 @@ _unur_stdgen_sample_beta_b00(  struct unur_gen *gen )
 #undef p1
 #undef p2
 /*---------------------------------------------------------------------------*/
+#define GEN_N_PARAMS (11)
 #define pint    (GEN->gen_param[0])
 #define qint    (GEN->gen_param[1])
 #define p_      (GEN->gen_param[2])
@@ -535,9 +548,9 @@ beta_b01_init( struct unur_gen *gen )
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
 
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
 
   /* -X- setup code -X- */
@@ -584,8 +597,8 @@ _unur_stdgen_sample_beta_b01(  struct unur_gen *gen )
   double U, V, X, Z;
 
   /* check arguments */
-  CHECK_NULL(gen,INFINITY);
-  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
 
   while (1) {
     U = uniform() * p2;
@@ -626,6 +639,7 @@ _unur_stdgen_sample_beta_b01(  struct unur_gen *gen )
 } /* end of _unur_stdgen_sample_beta_b01() */
 
 /*---------------------------------------------------------------------------*/
+#undef GEN_N_PARAMS
 #undef pint
 #undef qint
 #undef p_
@@ -638,6 +652,7 @@ _unur_stdgen_sample_beta_b01(  struct unur_gen *gen )
 #undef p1
 #undef p2
 /*---------------------------------------------------------------------------*/
+#define GEN_N_PARAMS (22)
 #define p_      (GEN->gen_param[0])
 #define q_      (GEN->gen_param[1])
 #define s       (GEN->gen_param[2])
@@ -670,9 +685,9 @@ beta_b1prs_init( struct unur_gen *gen )
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
 
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
 
   /* -X- setup code -X- */
@@ -742,8 +757,8 @@ _unur_stdgen_sample_beta_b1prs(  struct unur_gen *gen )
   double U, V, W, X, Y;
 
   /* check arguments */
-  CHECK_NULL(gen,INFINITY);
-  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
 
   while (1) {
     U = uniform() * p4;
@@ -855,6 +870,7 @@ _unur_stdgen_sample_beta_b1prs(  struct unur_gen *gen )
 } /* end of _unur_stdgen_sample_beta_b1prs() */
 
 /*---------------------------------------------------------------------------*/
+#undef GEN_N_PARAMS
 #undef p_
 #undef q_
 #undef s 
@@ -878,3 +894,31 @@ _unur_stdgen_sample_beta_b1prs(  struct unur_gen *gen )
 #undef p3
 #undef p4
 /*---------------------------------------------------------------------------*/
+double 
+_unur_stdgen_sample_beta_binv(  struct unur_gen *gen )
+     /* p == 1. || q == 1. */ 
+{
+  /* -X- generator code -X- */
+  double X;
+
+  /* check arguments */
+  CHECK_NULL(gen,UNUR_INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_INFINITY);
+
+  if (_unur_isone(p) && _unur_isone(q)) {
+    X = uniform();
+  }
+  else if (_unur_isone(p)) {
+    X = 1. - pow(1.-uniform(), 1/q);
+  }
+  else { /* _unur_isone(q) */
+    X = pow(uniform(), 1/p);
+  }
+
+  /* -X- end of generator code -X- */
+
+  return ((DISTR.n_params==2) ? X : a + (b-a) * X);
+
+} /* end of _unur_stdgen_sample_beta_binv() */
+
+/*---------------------------------------------------------------------------*/
