@@ -63,15 +63,9 @@
 
 /*---------------------------------------------------------------------------*/
 
+#include <unur_source.h>
 #include <unuran.h>
-#include <config.h>
-#include <unuran_config.h>
-#include <distr/distr_struct.h> 
 #include <utils/matrix_source.h> 
-#include <utils/unur_fp_source.h>
-#include <utils/unur_math_source.h>
-#include <specfunct/unur_specfunct_source.h>
-#include <float.h>
 #include "testdistributions.h"
 
 /*---------------------------------------------------------------------------*/
