@@ -10,7 +10,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2000-2006 Wolfgang Hoermann and Josef Leydold             *
+ *   Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold             *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -68,6 +68,9 @@ static FILE *TESTLOG = NULL;
 
 /*---------------------------------------------------------------------------*/
 
+/* draw sample */
+static int draw_sample_and_discard ( FILE *LOG, int line, UNUR_GEN *gen, int sample_size );
+
 /* compare two doubles */
 static int compare_doubles ( double x1, double x2 );
 
@@ -479,6 +482,48 @@ int check_expected_no_reinit( FILE *LOG, int line, int rcode )
   }
 } /* end of check_expected_no_reinit() */
 
+
+/*---------------------------------------------------------------------------*/
+/* draw sample */
+
+int draw_sample_and_discard ( FILE *LOG, int line, UNUR_GEN *gen, int sample_size )
+{
+  /* int J; */
+  /* double X; */
+  double *vec;
+
+  switch (unur_distr_get_type(unur_get_distr(gen)) ) {
+
+  case UNUR_DISTR_DISCR:
+    for( ; sample_size>0; --sample_size) 
+      /* J = unur_sample_discr(gen); */
+      unur_sample_discr(gen);
+    return UNUR_SUCCESS;
+
+  case UNUR_DISTR_CONT:
+  case UNUR_DISTR_CEMP:
+    for( ; sample_size>0; --sample_size) 
+      /* X = unur_sample_cont(gen); */
+      unur_sample_cont(gen);
+    return UNUR_SUCCESS;
+
+  case UNUR_DISTR_CVEC:
+  case UNUR_DISTR_CVEMP:
+    /* we need an array for the vector */
+    vec = malloc( unur_get_dimension(gen) * sizeof(double) );
+    abort_if_NULL(LOG, line, vec);
+    for( ; sample_size>0; --sample_size) 
+      unur_sample_vec(gen,vec);
+    free(vec);
+    return UNUR_SUCCESS;
+
+  default: /* unknown ! */
+    fprintf(stderr,"\ncannot handle distribution type! ... aborted\n");
+    exit (EXIT_FAILURE);
+  }
+
+} /* end of unur_test_printsample() */
+
 /*---------------------------------------------------------------------------*/
 /* compare two doubles */
 
@@ -572,7 +617,10 @@ compare_sequence_par_start( FILE *LOG, int line, UNUR_PAR *par, int sample_size
 
   /* reset uniform RNG */
   if (unur_urng_reset(unur_get_default_urng()) != UNUR_SUCCESS) {
-    if (par) free(par); return UNUR_SUCCESS;
+    if (par) {
+      free(par);
+    }
+    return UNUR_SUCCESS;
   }
 
   /* init generator */
@@ -594,7 +642,10 @@ compare_sequence_par ( FILE *LOG, int line, UNUR_PAR *par, int sample_size )
 
   /* reset uniform RNG */
   if (unur_urng_reset(unur_get_default_urng()) != UNUR_SUCCESS) {
-    if (par) free(par); return cannot_compare_sequence(LOG);
+    if (par) {
+      free(par);
+    }
+    return cannot_compare_sequence(LOG);
   }
 
   /* init generator */
@@ -1218,6 +1269,17 @@ int run_validate_chi2( FILE *LOG, int line ATTRIBUTE__UNUSED,
     }
   }
 
+  if (todo == '%') {
+    /* initialize generator and draw small sample.          */
+    /* (for memory debugging when the CDF is not available) */
+    printf("%%");  fflush(stdout);
+    draw_sample_and_discard(LOG, line, gen, 100);
+    fprintf(LOG,"   not performed (sample only)\t");
+    print_distr_name( LOG, distr, gen ? unur_get_genid(gen):"???\t" );
+    fprintf(LOG,"\n");
+    return UNUR_SUCCESS;
+  }
+
   /* init successful */
   if ( todo == '/' ) todo = '+';
 
